//____________________________________________________________
//    AsynchSndPlay.c
//
//    Copyright  Dan Parks Sydow, 1995
//    From the book:
//    "Graphics and Sound Programming Techniques for the Mac",
//    M&T Books, 1995
 
 
//____________________________________________________________

#include <Sound.h>


//____________________________________________________________

pascal void    SoundChannelCallback( SndChannelPtr, SndCommand );
OSErr          InstallCallbackCommand( SndChannelPtr );
void           CleanUpSoundIfFinished( void );
void           PlaySoundResourceAsynch( short );
void           AnimateWhileSoundPlays( void );
void           InitializeToolbox( void );
void           OpenDisplayWindow( void );
void           LoadAndSetupPicture( void );
void           MovePictureOnePixel( void );
SndChannelPtr  OpenOneAsynchSoundChannel( void );
OSErr          DisposeOneSoundChannel( SndChannelPtr );


//____________________________________________________________

#define      rMusicSound          9000
#define      rDisplayWindow        128
#define      rBearPicture          128


//____________________________________________________________

short          gSoundPlaying     = false;
Boolean        gCallbackExecuted = false;
SndChannelPtr  gSoundChannel     = nil;
Handle         gSoundHandle      = nil;
Boolean        gDone             = false;
PicHandle      gThePicture;
Rect           gTheRect;


//____________________________________________________________

void  main( void )
{
   NumVersion   theSndMgrVers;      
   EventRecord  theEvent;

   InitializeToolbox();
   
   theSndMgrVers = SndSoundManagerVersion();   
   if ( theSndMgrVers.majorRev < 3 )
      ExitToShell();
   
   OpenDisplayWindow();   
   LoadAndSetupPicture(); 

   while ( gDone == false )
   {
      WaitNextEvent( everyEvent, &theEvent, 15L, nil );
    
      switch ( theEvent.what )
      {
         case mouseDown:
            gDone = true;
            break;
          
         case keyDown:
            PlaySoundResourceAsynch( rMusicSound );
            break;
      }
   } 
}
   

//____________________________________________________________

void  PlaySoundResourceAsynch( short theResID )
{
   OSErr  theError;
   
   gSoundChannel = OpenOneAsynchSoundChannel();
   if ( gSoundChannel == nil )
      ExitToShell();
      
   gSoundHandle = GetResource( 'snd ', theResID );
   if ( gSoundHandle == nil )
      ExitToShell();
   
   DetachResource( gSoundHandle );      
   HLock( gSoundHandle );
   
   gSoundPlaying = true;
   
   theError = SndPlay( gSoundChannel, (SndListHandle)gSoundHandle, true );
      
   if ( theError == noErr )
      theError = InstallCallbackCommand( gSoundChannel );
   else
      ExitToShell();

   AnimateWhileSoundPlays();
}
  

//____________________________________________________________

void  AnimateWhileSoundPlays( void )
{
   Boolean  loopDone = false;
   
   while ( loopDone == false )
   {
      CleanUpSoundIfFinished();
      
      if ( gSoundPlaying == true )
         MovePictureOnePixel();
      else
         loopDone = true;
   } 
}


//____________________________________________________________

void  CleanUpSoundIfFinished( void )
{
   OSErr  theError;
   
   if ( gCallbackExecuted == true )
   {
      HUnlock( gSoundHandle );
      ReleaseResource( gSoundHandle );
      gSoundHandle = nil;
      
      theError = DisposeOneSoundChannel( gSoundChannel );
      if ( theError != noErr )
         ExitToShell();
         
      gSoundChannel = nil;
      gCallbackExecuted = false;
   }
}


//____________________________________________________________

OSErr  InstallCallbackCommand( SndChannelPtr theChannel )
{
   OSErr       theError;
   SndCommand  theCommand;
   
   theCommand.cmd    = callBackCmd;
   theCommand.param1 = 0;
   #ifndef powerc
      theCommand.param2 = SetCurrentA5();
   #else
      theCommand.param2 = 0;
   #endif
   
   theError = SndDoCommand( theChannel, &theCommand, true );
   
   return ( theError );
}


//____________________________________________________________

pascal void  SoundChannelCallback( SndChannelPtr theChannel, SndCommand theCommand )
{
   long  theA5;
   
   #ifndef powerc
      theA5 = SetA5( theCommand.param2 );
   #endif
      
   gCallbackExecuted = true;
   gSoundPlaying = false;
     
   #ifndef powerc
      theA5 = SetA5( theA5 );
   #endif
}



//____________________________________________________________

SndChannelPtr  OpenOneAsynchSoundChannel( void )
{
   SndChannelPtr   theChannel;   
   OSErr           theError;
   SndCallBackUPP  theCallBackUPP;

   theCallBackUPP = NewSndCallBackProc( SoundChannelCallback );
   
   theChannel = nil;
   theError = SndNewChannel( &theChannel, 0, 0, theCallBackUPP );
   
   if ( theError != noErr )
   {
      DisposePtr( (Ptr)theChannel );
      theChannel = nil;
   }
      
   return ( theChannel );
}


//____________________________________________________________

OSErr  DisposeOneSoundChannel( SndChannelPtr theChannel )
{
   OSErr  theError;
   
   theError = SndDisposeChannel( theChannel, true );
   DisposePtr( (Ptr)theChannel );
   
   return ( theError );
}


//____________________________________________________________

void  InitializeToolbox( void )
{
   InitGraf( &qd.thePort );
   InitFonts();
   InitWindows();
   InitMenus();
   TEInit();
   InitDialogs( 0L );
   FlushEvents( everyEvent, 0 );
   InitCursor();
}


//____________________________________________________________

void  OpenDisplayWindow( void )
{
   WindowPtr  theWindow;

   theWindow = GetNewWindow( rDisplayWindow, nil, (WindowPtr)-1L );
   ShowWindow( theWindow );
   SetPort( theWindow );
}


//____________________________________________________________

void  LoadAndSetupPicture( void )
{
   short  theWidth;
   short  theHeight;
   short  left = 475;
   short  top = 10;

   gThePicture = GetPicture( rBearPicture );
   gTheRect = (**gThePicture).picFrame;
   theWidth = gTheRect.right - gTheRect.left;
   theHeight = gTheRect.bottom - gTheRect.top;
   SetRect( &gTheRect, left, top, left + theWidth, top + theHeight );
}   
    
 
//____________________________________________________________

void  MovePictureOnePixel( void )
{
   OffsetRect( &gTheRect, -1, 0 );
   DrawPicture( gThePicture, &gTheRect );
}
