//____________________________________________________________
//    SoundHandle.c
//
//    Copyright  Dan Parks Sydow, 1995
//    From the book:
//    "Graphics and Sound Programming Techniques for the Mac",
//    M&T Books, 1995


//____________________________________________________________

#include <Sound.h>
#include <SoundInput.h>


//____________________________________________________________

void     InitializeToolbox( void );
Boolean  IsSoundInputAvailable( void );
OSErr    RecordSoundToMemory( SndListHandle * );
OSErr    PlaySoundSynchFromHandle( SndListHandle );


//____________________________________________________________

#define    kHeapReserve        75 * 1024 


//____________________________________________________________

void  main( void )
{
   NumVersion     theSndMgrVers;
   OSErr          theError;
   SndListHandle  theSound;
   Boolean        soundInputPresent;   
   
   InitializeToolbox();

   MaxApplZone();

   theSndMgrVers = SndSoundManagerVersion();   
   if ( theSndMgrVers.majorRev < 3 )
      ExitToShell();

   soundInputPresent = IsSoundInputAvailable();
   if ( soundInputPresent == false )
      ExitToShell();

   theError = RecordSoundToMemory( &theSound );
   if ( theError == userCanceledErr )
      ExitToShell();
      
   theError = PlaySoundSynchFromHandle( theSound );   
   if ( theError != noErr )
      ExitToShell();
      
   ReleaseResource( (Handle)theSound );
}


//____________________________________________________________

OSErr  RecordSoundToMemory( SndListHandle *theSound )
{
   OSErr  theError;
   Point  theCorner = { 50, 20 }; 
   long   theTotalHeap;
   long   theContigMem;
   
   PurgeSpace( &theTotalHeap, &theContigMem );

   *theSound = (SndListHandle)NewHandle( theContigMem - kHeapReserve );
   
   theError = SndRecord( nil, theCorner, siBestQuality, theSound );
   
   return ( theError );
}


//____________________________________________________________

OSErr  PlaySoundSynchFromHandle( SndListHandle theHandle )
{
   OSErr   theError;
   
   if ( theHandle != nil )
   {
      HLock( (Handle)theHandle );
         theError = SndPlay( nil, theHandle, false );
      HUnlock( (Handle)theHandle );

      return ( theError );
   }
}


//____________________________________________________________

Boolean  IsSoundInputAvailable( void )
{
   OSErr    theError;
   long     theResult;
   Boolean  inputAvail;
   
   theError = Gestalt( gestaltSoundAttr, &theResult );
   if ( theError != noErr )
      ExitToShell();
      
   inputAvail = theResult & ( 1 << gestaltHasSoundInputDevice );  
   if ( inputAvail > 0 )
      return ( true );
   else
      return ( false );
}


//____________________________________________________________

void  InitializeToolbox( void )
{
   InitGraf( &qd.thePort );
   InitFonts();
   InitWindows();
   InitMenus();
   TEInit();
   InitDialogs( 0L );
   FlushEvents( everyEvent, 0 );
   InitCursor();
}
