//____________________________________________________________
//    SoundSave.c
//
//    Copyright  Dan Parks Sydow, 1995
//    From the book:
//    "Graphics and Sound Programming Techniques for the Mac",
//    M&T Books, 1995


//____________________________________________________________

#include <Sound.h>
#include <SoundInput.h>


//____________________________________________________________

void     InitializeToolbox( void );
Boolean  IsSoundInputAvailable( void );
OSErr    RecordSoundToMemory( SndListHandle * );
long     SaveSoundFromMemoryToResource( SndListHandle );
OSErr    PlaySoundResourceSynch( short );


//____________________________________________________________

#define    kHeapReserve              75 * 1024
#define    kSndResIDMaxReserved      8191


//____________________________________________________________

void  main( void )
{
   NumVersion     theSndMgrVers;
   OSErr          theError;
   SndListHandle  theSound;
   long           theNewSoundID;
   Boolean        soundInputPresent;   
   
   InitializeToolbox();
   
   MaxApplZone();

   theSndMgrVers = SndSoundManagerVersion();   
   if ( theSndMgrVers.majorRev < 3 )
      ExitToShell();

   soundInputPresent = IsSoundInputAvailable();
   if ( soundInputPresent == false )
      ExitToShell();
      
   theError = RecordSoundToMemory( &theSound );
   if ( theError == userCanceledErr )
      ExitToShell();

   theNewSoundID = SaveSoundFromMemoryToResource( theSound );

   ReleaseResource( (Handle)theSound );
   theSound = nil;
   
   theError = PlaySoundResourceSynch( theNewSoundID );   
   if ( theError != noErr )
      ExitToShell();
}


//____________________________________________________________

OSErr  RecordSoundToMemory( SndListHandle *theSound )
{
   OSErr  theError;
   Point  theCorner = { 50, 20 }; 
   long   theTotalHeap;
   long   theContigMem;
   
   PurgeSpace( &theTotalHeap, &theContigMem );

   *theSound = (SndListHandle)NewHandle( theContigMem - kHeapReserve );
   
   theError = SndRecord( nil, theCorner, siBestQuality, theSound );
   
   return ( theError );
}


//____________________________________________________________

long  SaveSoundFromMemoryToResource( SndListHandle theSound )
{
   long   theResID;
   OSErr  theError;
   short  theResourceFileRef;

   theResourceFileRef = CurResFile();
   
   do
   {
      theResID = UniqueID( 'snd ' );
   } while ( theResID <= kSndResIDMaxReserved );

   AddResource( (Handle)theSound, 'snd ', theResID, "\pNew Sound" );
   theError = ResError();
   if ( theError != noErr )
      ExitToShell();
      
   UpdateResFile( theResourceFileRef );
   theError = ResError();
   if ( theError != noErr )
      ExitToShell();
   
   return ( theResID );
}


//____________________________________________________________

OSErr  PlaySoundResourceSynch( short theResID )
{
   Handle  theHandle;
   OSErr   theError;
   
   theHandle = GetResource( 'snd ', theResID );
   
   if ( theHandle == nil )
   {   
      return ( resProblem );
   }
   else
   {
      HLock( theHandle );
         theError = SndPlay( nil, (SndListHandle)theHandle, false );
      HUnlock( theHandle );
   
      ReleaseResource( theHandle );

      return ( theError );
   }
}


//____________________________________________________________

Boolean  IsSoundInputAvailable( void )
{
   OSErr    theError;
   long     theResult;
   Boolean  inputAvail;
   
   theError = Gestalt( gestaltSoundAttr, &theResult );
   if ( theError != noErr )
      ExitToShell();
      
   inputAvail = theResult & ( 1 << gestaltHasSoundInputDevice );  
   if ( inputAvail > 0 )
      return ( true );
   else
      return ( false );
}


//____________________________________________________________

void  InitializeToolbox( void )
{
   InitGraf( &qd.thePort );
   InitFonts();
   InitWindows();
   InitMenus();
   TEInit();
   InitDialogs( 0L );
   FlushEvents( everyEvent, 0 );
   InitCursor();
}
