//____________________________________________________________
//    GWorlds.c
//
//    Copyright  Dan Parks Sydow, 1995
//    From the book:
//    "Graphics and Sound Programming Techniques for the Mac",
//    M&T Books, 1995

//____________________________________________________________

#include <QDOffscreen.h>


//____________________________________________________________

void     InitializeToolbox( void );
void     OpenDisplayWindow( void );
Boolean  AreGWorldsAvailable( void );
void     SetForeAndBackColors( void );
void     CreateGWorldsAndPixMaps( void );
void     PrepareForAnimation( void );
void     AnimateOneFrame( void );
void     DetermineAnimationDirection( void );


//____________________________________________________________

#define       rBackPicture           128
#define       rForePicture           129
#define       rDisplayWindow         128
#define       kXincrement              1


//____________________________________________________________

WindowPtr     gDisplayWindow;
short         gCurrentXIncrement = kXincrement;
GWorldPtr     gForeGWorldPtr;
GWorldPtr     gBackGWorldPtr;
GWorldPtr     gMergeGWorldPtr;
PixMapHandle  gForePixMap;
PixMapHandle  gBackPixMap;
PixMapHandle  gMergePixMap;
Rect          gWindowPixMapRect;
Rect          gGWorldPixMapRect;


//____________________________________________________________

void	main( void )
{
   Boolean   graphicsWorldsPresent;
   Boolean   pixelsLocked;
   
   InitializeToolbox();

   HideCursor();
   
   graphicsWorldsPresent = AreGWorldsAvailable();
   if ( graphicsWorldsPresent == false )
      ExitToShell();

   SetForeAndBackColors();

   OpenDisplayWindow();
   	
   CreateGWorldsAndPixMaps();

   pixelsLocked = LockPixels( gForePixMap );
   if ( pixelsLocked == false )
      ExitToShell();

   pixelsLocked = LockPixels( gBackPixMap );
   if ( pixelsLocked == false )
      ExitToShell();

   pixelsLocked = LockPixels( gMergePixMap );
   if ( pixelsLocked == false )
      ExitToShell();
   
   PrepareForAnimation();
	
   while ( !Button() )
      AnimateOneFrame();

   UnlockPixels( gForePixMap );
   UnlockPixels( gBackPixMap );
   UnlockPixels( gMergePixMap );
   
   DisposeGWorld( gForeGWorldPtr );   
   DisposeGWorld( gBackGWorldPtr );   
   DisposeGWorld( gMergeGWorldPtr );   
}


//____________________________________________________________

Boolean  AreGWorldsAvailable( void )
{
   OSErr    theError;
   long     theResult;
   Boolean  worldAvail;
   
   theError = Gestalt( gestaltQuickdrawFeatures, &theResult );
   if ( theError != noErr )
      ExitToShell();
      
   worldAvail = theResult & ( 1 << gestaltHasDeepGWorlds );  
   if ( worldAvail > 0 )
      return ( true );
   else
      return ( false );
}


//____________________________________________________________

void SetForeAndBackColors( void )
{
   RGBColor  theBlackColor;
   RGBColor  theWhiteColor;

   theBlackColor.red   = 0x0000;
   theBlackColor.green = 0x0000;
   theBlackColor.blue  = 0x0000;
   
   theWhiteColor.red   = 0xFFFF;
   theWhiteColor.green = 0xFFFF;
   theWhiteColor.blue  = 0xFFFF;

   RGBForeColor( &theBlackColor );
   RGBBackColor( &theWhiteColor );
}


//____________________________________________________________

void  OpenDisplayWindow( void )
{
   PicHandle  theBackPicture;
   Rect       theRect;
	
   theBackPicture = GetPicture( rBackPicture );
   if ( theBackPicture == nil )
      ExitToShell();

   theRect = (**theBackPicture).picFrame;	
   OffsetRect( &theRect, - theRect.left, - theRect.top );

   gDisplayWindow = GetNewWindow( rDisplayWindow, nil, (WindowPtr)-1L );
   ShowWindow( gDisplayWindow );
   SetPort( gDisplayWindow );
	
   DrawPicture( theBackPicture, &theRect );
}


//____________________________________________________________

void  CreateGWorldsAndPixMaps( void )
{
   PicHandle  theForePicture;
   QDErr      theError;
   
   theForePicture = GetPicture( rForePicture );
   if ( theForePicture == nil )
      ExitToShell();

   gGWorldPixMapRect = (**theForePicture).picFrame;
   OffsetRect( &gGWorldPixMapRect, - gGWorldPixMapRect.left, - gGWorldPixMapRect.top );
		
   theError = NewGWorld( &gForeGWorldPtr, 0, &gGWorldPixMapRect, nil, nil, noNewDevice );
   theError = NewGWorld( &gBackGWorldPtr, 0, &gGWorldPixMapRect, nil, nil, noNewDevice );
   theError = NewGWorld( &gMergeGWorldPtr, 0, &gGWorldPixMapRect, nil, nil, noNewDevice );

   gForePixMap = GetGWorldPixMap( gForeGWorldPtr );
   gBackPixMap = GetGWorldPixMap( gBackGWorldPtr );
   gMergePixMap = GetGWorldPixMap( gMergeGWorldPtr );

   SetGWorld( gForeGWorldPtr, nil );
		
   DrawPicture( theForePicture, &gGWorldPixMapRect );
}


//____________________________________________________________

void  PrepareForAnimation( void )
{
   gWindowPixMapRect = gGWorldPixMapRect;
   OffsetRect( &gWindowPixMapRect, 0, 25 );

   CopyBits( &(gDisplayWindow->portBits), (BitMap *)(*gBackPixMap),
             &gWindowPixMapRect, &gGWorldPixMapRect, srcCopy, nil );
	
   CopyBits( (BitMap *)(*gBackPixMap), (BitMap *)(*gMergePixMap),
             &gGWorldPixMapRect, &gGWorldPixMapRect, srcCopy, nil );		
}


//____________________________________________________________

void  AnimateOneFrame( void )
{
   Rect  theTempRect;

   DetermineAnimationDirection();
	
   theTempRect = gGWorldPixMapRect;
   OffsetRect( &theTempRect, gCurrentXIncrement, 0 );
	
   CopyBits( (BitMap *)(*gForePixMap), (BitMap *)(*gMergePixMap),
             &gGWorldPixMapRect, &theTempRect, transparent, nil );
	
   CopyBits( (BitMap *)(*gMergePixMap), &(gDisplayWindow->portBits),
             &gGWorldPixMapRect, &gWindowPixMapRect, srcCopy, nil );

   OffsetRect( &gWindowPixMapRect, gCurrentXIncrement, 0 );
	
   CopyBits( &(gDisplayWindow->portBits), (BitMap *)(*gMergePixMap),
      &gWindowPixMapRect, &gGWorldPixMapRect, srcCopy, nil );
	
   theTempRect = gGWorldPixMapRect;
   OffsetRect( &theTempRect, -gCurrentXIncrement, 0 );
	
   CopyBits( (BitMap *)(*gBackPixMap), (BitMap *)(*gMergePixMap),
             &gGWorldPixMapRect, &theTempRect, srcCopy, nil );
	
   CopyBits( (BitMap *)(*gMergePixMap), (BitMap *)(*gBackPixMap),
             &gGWorldPixMapRect, &gGWorldPixMapRect, srcCopy, nil );
}


//____________________________________________________________

void  DetermineAnimationDirection( void )
{
   Rect  theRect;
	
   theRect = gWindowPixMapRect;
	
   OffsetRect( &theRect, gCurrentXIncrement, 0 );
	
   if ( theRect.right > gDisplayWindow->portRect.right )
      gCurrentXIncrement = - kXincrement;
   else if ( theRect.left < gDisplayWindow->portRect.left )
      gCurrentXIncrement = + kXincrement;
}


//____________________________________________________________

void  InitializeToolbox( void )
{
   InitGraf( &qd.thePort );
   InitFonts();
   InitWindows();
   InitMenus();
   TEInit();
   InitDialogs( 0L );
   FlushEvents( everyEvent, 0 );
   InitCursor();
}


