//____________________________________________________________
//    QDGXIntro.c
//
//    Copyright  Dan Parks Sydow, 1995
//    From the book:
//    "Graphics and Sound Programming Techniques for the Mac",
//    M&T Books, 1995

//____________________________________________________________


#include "PrintingManager.h"      // defines the Gestalt selector code gestaltGXPrintingMgrVersion
#include "graphics macintosh.h"   // defines the Gestalt selector code gestaltGraphicsVersion
#include "graphics libraries.h"   // prototypes for several GX functions


//____________________________________________________________

Boolean  IsQuickDrawGXAvailable( void );
void     InitializeToolbox( void );
void     InitializeQuickDrawGX( void );
void     OpenDisplayWindow( void );
void     CleanUpAndQuit( void );
void     CleanUpQuickDrawGXandQuit( void );


//____________________________________________________________

#define     kGXClientHeapSizeBytes    150 * 1024


//____________________________________________________________

gxGraphicsClient  gGXClient;
Boolean           gQuickDrawGXPresent;
WindowPtr         gDisplayWindow = nil;
Boolean           gDone = false;


//____________________________________________________________

void  main( void )
{
   InitializeToolbox();

   gQuickDrawGXPresent = IsQuickDrawGXAvailable();
   if ( gQuickDrawGXPresent == true )
      InitializeQuickDrawGX();

   OpenDisplayWindow();
         
   while ( gDone == false )
   {
      if ( Button() )
      {
         if ( gQuickDrawGXPresent == true )   
            CleanUpQuickDrawGXandQuit();
         else
            CleanUpAndQuit();
      }
   }      
}


//____________________________________________________________

Boolean  IsQuickDrawGXAvailable( void )
{
   OSErr  theError;
   long   theResult;
   
   theError = Gestalt( gestaltGraphicsVersion, &theResult );
   if ( theError != noErr )
      return ( false );

   theError = Gestalt( gestaltGXPrintingMgrVersion, &theResult );
   if ( theError != noErr )
      return ( false );
      
   return ( true );
}


//____________________________________________________________

void  InitializeQuickDrawGX( void )
{
   gxGraphicsError  theGXgraphicsError;
   OSErr            theGXprintError;      
   
   gGXClient = GXNewGraphicsClient( nil, kGXClientHeapSizeBytes, 0L );

   GXEnterGraphics();
   theGXgraphicsError = GXGetGraphicsError( nil );
   if ( theGXgraphicsError == out_of_memory )
      ExitToShell();

   theGXprintError = GXInitPrinting();
   if ( theGXprintError != noErr )
      ExitToShell();
}


//____________________________________________________________

void  OpenDisplayWindow( void )
{
   gDisplayWindow = GetNewCWindow( 128, nil, (WindowPtr)-1L );
   ShowWindow( gDisplayWindow );
   SetPort( gDisplayWindow );
   MoveTo( 20, 20 );

   if ( gQuickDrawGXPresent == true )   
      DrawString( "\pQuickDraw GX is present and enabled" );
   else
      DrawString( "\pQuickDraw GX not present or not enabled" );
}


//____________________________________________________________

void  CleanUpQuickDrawGXandQuit( void )
{
   OSErr  theGXprintError;      

   if ( gDisplayWindow != nil )
      DisposeWindow( gDisplayWindow );  
       
   theGXprintError = GXExitPrinting();

   GXExitGraphics();
   
   GXDisposeGraphicsClient( gGXClient ); 

   gDone = true;
}


//____________________________________________________________

void  CleanUpAndQuit( void )
{
   if ( gDisplayWindow != nil )
      DisposeWindow( gDisplayWindow );  
       
   gDone = true;
}


//____________________________________________________________

void  InitializeToolbox( void )
{
   InitGraf( &qd.thePort );
   InitFonts();
   InitWindows();
   InitMenus();
   TEInit();
   InitDialogs( 0L );
   FlushEvents( everyEvent, 0 );
   InitCursor();
}
